!---------------------------------------------c
!  Exercise: Pi                               c
!                                             c
!  Compute the value of PI using the integral c
!  pi = 4* int 1/(1+x*x)    x in [0-1]        c
!                                             c
!  The integral is approximated by a sum of   c
!  n interval.                                c
!                                             c
!  The approximation to the integral in each  c
!  interval is: (1/n)*4/(1+x*x).              c
!---------------------------------------------c
program integrate

  use iso_c_binding
  use omp_lib
  
  implicit none

  integer(C_LONG)            :: i 
  integer(C_LONG), parameter :: intervals = 4e8
  
  real(C_DOUBLE), parameter  :: dx = 1.d0/intervals
  real(C_DOUBLE)             :: sum,x
  real(C_DOUBLE)             :: f,pi
  
  real(C_DOUBLE),  parameter :: PI25DT = acos(-1.d0)
  real                       :: time(2)
  
  time(1) = omp_get_wtime()
  
  print *, 'Number of intervals: ', intervals
  sum = 0.d0
  
  !$OMP PARALLEL DO PRIVATE(x,f) REDUCTION(+:sum)
  do i=1,intervals
     x   = dx*(i-0.5d0)
     f   = 4.d0/(1.d0+x*x)
     sum = sum+f
  end do
  !$OMP END PARALLEL DO

  pi=dx*sum

  time(2) = omp_get_wtime()
  
  PRINT '(a13,2x,f30.25)',' Computed PI =', pi
  PRINT '(a13,2x,f30.25)',' The True PI =', PI25DT
  PRINT *, ' '
  PRINT *, 'Elapsed time ', time(2)-time(1) ,' s' 

end program integrate
